<?php
/**
 * Drwho controller for show characters
 * 
 * Accepts an array of configuration values to the constructor. If no config 
 * passed, it will automatically determine the component and controller names.
 * Internally, sets the $database, $user, $view, and component $config.
 * 
 * Executable tasks are determined by method name. All public methods that end in 
 * "Task" (e.g., displayTask, editTask) are callable by the end user.
 * 
 * View name defaults to controller name with layout defaulting to task name. So,
 * a $controller of "One" and a $task of "two" will map to:
 *
 *    /{component name}
 *        /views
 *            /one
 *                /tmpl
 *                    /two.php
 */
class DrwhoControllerCharacters extends Hubzero_Controller
{
	/**
	 * Default task. Displays a list of characters.
	 *
	 * @return	void
	 */
	public function displayTask()
	{
		// Get our model
		// This is the entry point to the database and the 
		// table of characters we'll be retrieving data from
		$this->view->model = new DrwhoModelShow();

		// NOTE:
		// A \Hubzero\Component\View object is auto-created when calling
		// execute() on the controller. By default, the view directory is 
		// set to the controller name and layout is set to task name.
		//
		// controller=foo&task=bar   loads a view from:
		//
		//   view/
		//     foo/
		//       tmpl/
		//         bar.php
		//
		// A new layout or name can be chosen by calling setLayout('newlayout')
		// or setName('newname') respectively.

		// Incoming filters
		$this->view->filters = array(
			'limit'  => JRequest::getInt('limit', 25),
			'start'  => JRequest::getInt('limitstart', 0),
			'season' => JRequest::getVar('season', '')
		);

		// If we have a set season...
		if ($this->view->filters['season'])
		{
			// Seasons are typically passed in the URL by their alias (text string)
			// but we need the season's numerical ID. So, we load the season's 
			// database record and retrieve it's ID.
			//
			// NOTE: We're using the with() function here to make the code a 
			// little more terse. This function accepts an object and immediately 
			// returns it. This is handy for chainging.
			$season = new DrwhoModelSeason($this->view->filters['season']);
			$this->view->filters['season'] = $season->get('id');
		}

		// Get a total record count
		// This is used for pagination and detemrining the total number of pages
		$this->view->total   = $this->view->model->characters('count', $this->view->filters);

		// Get a list of records
		$this->view->records = $this->view->model->characters('list', $this->view->filters);

		// Output the view
		// 
		// Make sure we load the correct view. This is for cases where 
		// we may be redirected from editTask(), which can happen if the
		// user is not logged in.
		$this->view->setLayout('display');
		$this->view->display();
	}

	/**
	 * Display a form for editing or creating a character entry.
	 *
	 * @return	void
	 */
	public function viewTask()
	{
		// Grab the incoming ID and load the record for viewing
		$id = JRequest::getInt('id', 0);

		$this->view->character = new DrwhoModelCharacter($id);

		// Make sure the entry exists. If it doesn't, throw
		// a 404 error. This will display the error layout
		// defined in the active template (if defined at all).
		if (!$this->view->character->exists())
		{
			throw new JException(JText::_('COM_DRWHO_CHARACTER_NOT_FOUND'), 404);
		}

		// Output the view
		$this->view->display();
	}

	/**
	 * Display a form for creating an entry.
	 *
	 * @return  void
	 */
	public function addTask()
	{
		$this->editTask();
	}

	/**
	 * Display a form for editing or creating an entry.
	 *
	 * @param   object  $tbl  DrwhoTableCharacter
	 * @return  void
	 */
	public function editTask($tbl=null)
	{
		// NOTE:
		// The JFactory::getUser() object is auto-assigned to $this->juser 
		// when calling execute() on the controller.

		// Only logged in users!
		if ($this->juser->get('guest'))
		{
			return $this->displayTask();
		}

		// If we're being passed an object, use it instead
		// Thsi means we came from saveTask() and some error occurred.
		// Most likely a missing or incorrect field.
		if ($tbl instanceof DrwhoModelCharacter)
		{
			$this->view->entry = $tbl;
		}
		else
		{
			// Grab the incoming ID and load the record for editing
			$id = JRequest::getInt('id', 0);

			$this->view->entry = new DrwhoModelCharacter($id);
		}

		// Get the show model.
		// We will need this in the form to output a list of seasons.
		$this->view->model = new DrwhoModelShow();

		// Pass any received errors to the view
		// These will be coming from the editTask()
		if ($this->getError())
		{
			foreach ($this->getErrors() as $error)
			{
				$this->view->setError($error);
			}
		}

		// Output the view
		// 
		// Make sure we load the edit view.
		// This is for cases where saveTask() might encounter a data
		// validation error and fall through to editTask(). Since layout 
		// is auto-assigned the task name, the layout will be 'save' but
		// saveTask has no layout!
		$this->view->setLayout('edit');
		$this->view->display();
	}

	/**
	 * Save a character entry to the database and redirect back to
	 * the main view
	 *
	 * @return  void
	 */
	public function saveTask()
	{
		// [SECURITY] This is a Cross-Site Request Forgery token check
		JRequest::checkToken() or jexit('Invalid token');

		// Incoming data, specifically from POST
		$data = JRequest::getVar('entry', array(), 'post', 2, 'none');

		// Bind the incoming data to our mdoel
		//
		// We're extending \Hubzero\Base\Model, which means the 
		// constructor, by default, accepts an integer, array, or object.
		//
		// - integer: load a record from the database with the 
		//            ID (primary key) of that value
		//
		// - array or object: bind the data to the inernal JTable object
		//
		$model = new DrwhoModelCharacter($data);

		// Pass the data to any plugins that be respond to the 
		// onBeforeSave event. This is where extra data manipulation 
		// or validation can be done.
		JPluginHelper::importPlugin('drwho');
		$dispatcher = JDispatcher::getInstance();
		$dispatcher->trigger('onBeforeSave', array($model));

		// Validate and save the data
		//
		// The store() method of models accepts a boolean:
		// - TRUE if the data should be validated before saving.
		// - FALSE if the data should NOT be validated before saving.
		//
		// If store() returns false for any reason, me pass the error
		// message from the model to the controller and fall through
		// to the edit form. We pass the existing model to the edit form
		// so it can repopulate the form with the user-submitted data.
		if (!$model->store(true))
		{
			$this->setError($model->getError());
			$this->editTask($model);
			return;
		}

		// Get all the seasons this character has been assigned to.
		//
		// Here we're grabbing the array of seasons (checkboxes) and
		// assigning the model to each season. It will also remove any
		// previous assignments not found in the passed-in list.
		$seasons = JRequest::getVar('seasons', array(), 'post');
		if (!$model->assign($seasons))
		{
			$this->setError($model->getError());
			$this->editTask($model);
			return;
		}

		// Redirect back to the main listing with a success message
		$this->setRedirect(
			JRoute::_('index.php?option=' . $this->_option . '&controller=' . $this->_controller),
			JText::_('COM_DRWHO_RECORD_SAVED'),
			'passed'  // Accepts passed, warning, or error
		);
	}

	/**
	 * Remove an entry
	 *
	 * @return  void
	 */
	public function deleteTask()
	{
		// [SECURITY] This is a Cross-Site Request Forgery token check
		JRequest::checkToken('get') or jexit('Invalid token');

		// Incoming data, specifically from POST
		$id = JRequest::getInt('id', 0);

		// Bind the incoming data to our mdoel
		$model = new DrwhoModelCharacter($id);

		// Check that the record exists. If it doesn't, then
		// we're all done here.
		if (!$model->exists())
		{
			// Redirect back to the main listing
			$this->setRedirect(
				JRoute::_('index.php?option=' . $this->_option . '&controller=' . $this->_controller)
			);
			return;
		}

		// Remove the entry and associated data
		//
		// If the model fails to remove the entry, it will pass
		// an error message to the cotnroller. The controller 
		// will then pass the error message along with the redirect
		// to the default task. The CMS detects a message has been
		// set and displays it in the template.
		if (!$model->delete())
		{
			$this->setError($model->getError());
		}

		// Redirect back to the main listing
		$this->setRedirect(
			JRoute::_('index.php?option=' . $this->_option . '&controller=' . $this->_controller),
			($this->getError() ? $this->getError() : JText::_('COM_DRWHO_RECORD_DELETED')),
			($this->getError() ? 'error' : 'passed')
		);
	}
}